namespace Chapter15CSharp
{
	using System;
	using System.Collections;
	using System.ComponentModel;
	using System.Data;
	using System.Diagnostics;
	using System.Web;
	using System.Web.Services;
	using System.Web.Services.Protocols;
	using System.IO;
	using System.Xml;

	// Create a new custom attribute that will allow
	// users to configure the log file. This attribute
	// will also tell the ASP.NET system which type to
	// load for our extension
	[AttributeUsage(AttributeTargets.Method)]
	public class SoapLoggerAttribute : SoapExtensionAttribute
	{
		private int _priority = 0;
		private string _logFile = @"C:\soap.log";
		public SoapLoggerAttribute() {}
		public SoapLoggerAttribute(string logFile) {_logFile = logFile;}
		public string LogFile {get{return _logFile;}
		}
		public override Type ExtensionType {get{return typeof(SoapLogger);}
		}
		public override int Priority {get {return _priority;} set{_priority = value;}
		}
	}

	// Create a new SoapExtension class. When users
	// apply the above attribute to a method of their 
	// web service, this class will be instantiated
	// by the ASP.NET system
	public class SoapLogger : SoapExtension
	{
		private string LogFile = "";
		private Stream SoapStream;
		private Stream TempStream;

		public override object GetInitializer(Type serviceType)
		{
			// we need to read some service configuration from the attribute
			// so differ creating an initializer
			return null;
		}
		public override object GetInitializer(LogicalMethodInfo methodInfo, SoapExtensionAttribute attribute)
		{
			// whatever we return here will be passed back to
			// our initialize method every time the ASP.NET system
			// needs to create a new instance of this extension class
			return ((SoapLoggerAttribute)attribute).LogFile;
		}
		public override void Initialize(object initializer)
		{
			// grab the logfile name that we returned in
			// GetInitializer
			LogFile = (string)initializer;
		}
		public override Stream ChainStream(Stream stream)
		{
			// by overriding ChainStream we can
			// cause the ASP.NET system to use
			// our stream for buffering SOAP messages
			// rather than the default stream.
			// we will store off the original stream
			// so we can pass the data back down to the ASP.NET system
			// in original stream that it created.
			SoapStream = stream;
			TempStream = new MemoryStream();
			return TempStream;
		}
		public void CopyTextStream(Stream src, Stream dest)
		{
			TextReader reader = new StreamReader(src);
			TextWriter writer = new StreamWriter(dest);
			writer.WriteLine(reader.ReadToEnd());
			writer.Flush();
		}
		public override void ProcessMessage(SoapMessage message)
		{
			// this method will be called several times during
			// the processing of a SOAP request. The ASP.NET system
			// tells us which stage the SOAP request is at with the 
			// Stage property of the SoapMessage class
			switch (message.Stage)
			{
				case SoapMessageStage.BeforeDeserialize:
				{
					// copy the SOAP request from the network stream
					// into our memory buffer
					CopyTextStream(SoapStream, TempStream);
					FileStream fs = new FileStream(LogFile, FileMode.Append, FileAccess.Write);
					StreamWriter sw = new StreamWriter(fs);
					sw.WriteLine("** BEGIN SOAP REQUEST: {0}", DateTime.Now);
					sw.Flush();

					// copy the mem buffer stream to the log file
					TempStream.Position = 0;
					CopyTextStream(TempStream, fs);
					sw.WriteLine("** END SOAP REQUEST");
					sw.Flush();
					fs.Close();

					// reset the memory buffer position
					// so the ASP.NET system can parse and
					// decode the message
					TempStream.Position = 0;
				}
					break;
				case SoapMessageStage.AfterSerialize:
				{
					FileStream fs = new FileStream(LogFile, FileMode.Append, FileAccess.Write);
					StreamWriter sw = new StreamWriter(fs);
					sw.WriteLine("** BEGIN SOAP RESPONSE: {0}", DateTime.Now);
					sw.Flush();
					TempStream.Position = 0;
					CopyTextStream(TempStream, fs);
					sw.WriteLine("** END SOAP RESPONSE");
					sw.Flush();

					// copy the memory beffered response 
					// to the network stream
					TempStream.Position = 0;
					CopyTextStream(TempStream, SoapStream);

					fs.Close();
				}
					break;
			}

			return;
		}
	}
}